/*
 * File:	freedom.h
 * Purpose:	Kinetis Freedom hardware definitions
 *
 * Notes:
 */

#ifndef __FREEDOM_H__
#define __FREEDOM_H__

#include "stdio.h"
#include "Kinetis.h"
#include "mcg.h"

/***********************************************************************/
/*
 * The basic data types
 */
typedef unsigned char		uint8;	/*  8 bits */
typedef unsigned short int	uint16; /* 16 bits */
typedef unsigned long int	uint32; /* 32 bits */

typedef char			int8;	/*  8 bits */
typedef short int		int16;	/* 16 bits */
typedef int			int32;	/* 32 bits */

typedef volatile int8		vint8;	/*  8 bits */
typedef volatile int16		vint16; /* 16 bits */
typedef volatile int32		vint32; /* 32 bits */

typedef volatile uint8		vuint8;	 /*  8 bits */
typedef volatile uint16		vuint16; /* 16 bits */
typedef volatile uint32		vuint32; /* 32 bits */

/********************************************************************/

// Define which CPU  you are using.
#define CPU_MKL25Z128LK4

/*
* Input Clock Info
*/
#define CLK0_FREQ_HZ	    8000000
#define CLK0_TYPE	    CRYSTAL

/*
 * PLL Configuration Info
 */
//#define NO_PLL_INIT  // define this to skip PLL initilaization and stay in default FEI mode

/* The expected PLL output frequency is:
 * PLL out = (((CLKIN/PRDIV) x VDIV) / 2)
 * where the CLKIN is CLK0_FREQ_HZ.
 *
 * For more info on PLL initialization refer to the mcg driver files.
 */

#define PLL0_PRDIV	4	// divider eference by 4 = 2 MHz
#define PLL0_VDIV	48	// multiply reference by 48 = 96 MHz
//#define PLL0_VDIV	  24	  // multiply reference by 24 = 48 MHz

/* Serial Port Info */
/**************************************************************************
   * Note:
   *
   * Because of the changes to the UART modules, we can no longer define
   * the TERM_PORT as a base pointer.  The uart functions have been modified
   * accommodate this change.  Now, TERM_PORT_NUM must be defined as the
   * number of the UART port desired to use
   *
   * TERM_PORT_NUM = 0	-- This allows you to use UART0; default pins are
   *						PTA14 and PTA15
   *
   * TERM_PORT_NUM = 1	-- This allows you to use UART1; default pins are
   *						PTC3 and PTC4
   *
   * TERM_PORT_NUM = 2	-- This allows you to use UART2; default pins are
   *						PTD2 and PTD3
   *
   *************************************************************************/

/* Uses UART0 for OpenSDA */

#define TERM_PORT_NUM	    0

#define TERMINAL_BAUD	    19200
#undef	HW_FLOW_CONTROL

extern uint16 ti_print;
extern uint16 ti_accel_sampling;
extern uint16 ti_task;
extern uint16 ti_console;
extern uint16 ti_fall;
extern uint16 ti_delay;
extern uint16 base_time125m;

#endif /* __FREEDOM_H__ */
